import { Subscription, combineLatest, Subject, from } from 'rxjs';
import { switchMap, share, map, distinctUntilChanged, publishReplay, refCount } from 'rxjs/operators/index.js';
import { createConnection, generateContributionId } from '@adminide-stack/extension-api';
import { createServices } from '../services/services.js';
import { config } from '../config/env-config.js';
import { createExtensionHostClient } from './extension-client.js';
import { isErrorLike } from '../util/errors.js';
import { isDefined } from '../util/types.js';
import { parseContributionExpressions } from '../services/contribution-service.js';

/**
 * Creates the controller, which handles all communication between the client application and extensions.
 *
 * There should only be a single controller for the entire client application. The controller's model represents
 * all of the client application state that the client needs to know.
 */
function createController(context, internalService) {
  const subscriptions = new Subscription();
  // Log context to check if it's correctly passed
  if (!context) {
    console.error('Error: context is undefined or null!');
    return null;
  }
  // Initialize services, logging potential issues
  let services;
  try {
    services = createServices(context, internalService);
  } catch (error) {
    console.error(error, 'Error initializing Services:');
    return null;
  }
  const extensionHostConnection = combineLatest([context.createExtensionHost().pipe(switchMap(async messageTransports => {
    const connection = createConnection(messageTransports);
    connection.listen();
    const organization = await internalService.organizationContextService.getOrganization();
    const initData = {
      cdeopsURL: context.cdeopsURL,
      clientApplication: context.clientApplication,
      graphqlSubscriptionUrl: config.GRAPHQL_SUBSCRIPTION_URL,
      extensionSocketUrl: config.EXTENSION_SOCKET_URL,
      organization: {
        id: organization.id,
        name: organization.name,
        resources: organization.resources,
        configuration: organization.configuration
      }
    };
    await connection.sendRequest('initialize', [initData]);
    return connection;
  }), share()), context.traceExtensionHostCommunication]).pipe(
  // tap(([connection, trace]) => connection.trace(trace ? logger : null)),
  map(([connection]) => connection), distinctUntilChanged());
  const client = createExtensionHostClient(services, extensionHostConnection);
  subscriptions.add(client);
  subscriptions.add(registerExtensionContributions(services.contribution, services.extensions));
  const notifications = new Subject();
  return {
    notifications,
    services,
    unsubscribe: () => subscriptions.unsubscribe()
  };
}
function registerExtensionContributions(contributionRegistry, {
  activeExtensions
}) {
  const contributions = from(activeExtensions).pipe(map(extensions => extensions.map(({
    manifest
  }) => manifest).filter(manifest => manifest !== null && !isErrorLike(manifest)).map(({
    contributes
  }) => contributes).filter(isDefined).map(contributions => {
    try {
      return parseContributionExpressions(contributions);
    } catch (error) {
      // An error during evaluation causes all of the contributions in the same entry to be
      // discarded.
      console.warn('Discarding contributions: parsing expressions or templates failed.', {
        contributions,
        error
      });
      return {};
    }
  })),
  // Perf optimization: only parse all the context expression once if there are multiple Subscribers.
  // This does not change the behaviour of the Observable, it always emits the current value on Subscription.
  publishReplay(1), refCount());
  // return contributionRegistry.registerContributions({ [id]: contributions } as any);
  const id = generateContributionId();
  return contributionRegistry.registerContributions({
    [id]: contributions
  });
}

export { createController, registerExtensionContributions };
//# sourceMappingURL=controller.js.map
