import 'reflect-metadata';
import { Subscription, Subject } from 'rxjs';
import gql from 'graphql-tag';
import { tryCatchPromise, createConnection, createProxy, handleRequests, handleServerProxyRequests, ConfigurationTarget as ConfigurationTarget$1, LogLevel, RelativePattern, createWebWorkerMessageTransports } from '@adminide-stack/extension-api';
import { mixin, deepClone } from '@vscode-alt/monaco-editor/esm/vs/base/common/objects.js';
import { URI } from '@vscode-alt/monaco-editor/esm/vs/base/common/uri.js';
import { ConfigurationTarget } from '@adminide-stack/extension-api/lib/core/types/ext-host-types.js';
import { OVERRIDE_PROPERTY_REGEX, ConfigurationChangeEvent, Configuration, ConfigurationModel, OrganizationContext, OrganizationResource } from '@adminide-stack/core';
import { ResourceMap, TernarySearchTree } from '@vscode-alt/monaco-editor/esm/vs/base/common/map.js';
import { isObject } from '@vscode-alt/monaco-editor/esm/vs/base/common/types.js';
import { Barrier } from '@vscode-alt/monaco-editor/esm/vs/base/common/async.js';
import { filter, map } from 'rxjs/operators/index.js';
import { delta as delta$1 } from '@vscode-alt/monaco-editor/esm/vs/base/common/arrays.js';
import { Emitter } from '@vscode-alt/monaco-editor/esm/vs/base/common/event.js';
import { isLinux } from '@vscode-alt/monaco-editor/esm/vs/base/common/platform.js';
import { basenameOrAuthority, relativePath, isEqual, dirname } from '@vscode-alt/monaco-editor/esm/vs/base/common/resources.js';
import { compare } from '@vscode-alt/monaco-editor/esm/vs/base/common/strings.js';
import { ApolloClient } from '@apollo/client/index.js';
import { SubscriptionClient } from 'subscriptions-transport-ws';
import { WebSocketLink } from '@apollo/client/link/ws/index.js';
import { InMemoryCache } from '@apollo/client/cache/index.js';
import { Counter } from '@vscode-alt/monaco-editor/esm/vs/base/common/numbers.js';
import { ExtHostContext as ExtHostContext$1 } from '@adminide-stack/extension-api/lib/protocol/server.protocol.js';
import { RPCProtocol } from '@adminide-stack/extension-api/lib/protocol/rpc-protocol.js';
import { RPCLogger } from '@adminide-stack/extension-api/lib/protocol/rpc-logger.js';
import { BrowserRemoteRPC } from '@adminide-stack/extension-api/lib/connections/remote-rpc/browser-remote-rpc.js';
import { logger } from '@cdm-logger/client';

/** @internal */
class ExtHostContext {
  proxy;
  constructor(proxy) {
    this.proxy = proxy;
  }
  updateContext(updates) {
    this.proxy.$acceptContextUpdates(updates);
  }
}

class WebExtHostExtensionService {
  /** Extension's deactivate functions. */
  extensionDeactivate = new Map();
  /**
   * Proxy method invoked by the client to load an extension and invoke its `activate` function to start running it.
   *
   * It also sets up global hooks so that when the extension's code uses `require('cdeops/client')` and
   * `import 'cdeops/client'`, it gets the extension API handle (the value specified in
   * cdeops.d.ts).
   *
   * @param extensionID The extension ID of the extension to activate.
   * @param bundleURL The URL to the JavaScript source file (that exports an `activate` function) for
   * the extension.
   * @returns A promise that resolves when the extension's activation finishes (i.e., when it returns if it's synchronous,
   * or when the promise it returns resolves if it's async)
   */
  async $activateExtension(extensionID, bundleURL) {
    // Load the extension bundle and retrieve the extension entrypoint module's exports on
    // the global `module` property.
    try {
      self.exports = {};
      self.module = {};
      self.importScripts(bundleURL);
      console.log("SELF.MODULE: ", self.module);
    } catch (err) {
      throw new Error(`error thrown while executing extension ${JSON.stringify(extensionID)}
                bundle (in importScripts of ${bundleURL}): ${err}`);
    }
    const extensionExports = self.module.exports || {};
    delete self.exports;
    delete self.module;
    if (!extensionExports || !('activate' in extensionExports)) {
      throw new Error(`extension bundle for ${JSON.stringify(extensionID)} has no exported activate
                function (in ${bundleURL})`);
    }
    // During extension deactivation, we fist call the extension's `deactivate` function and then unsubscribe
    // the Subscription passed to the `activate` function.
    const extensionSubscriptions = new Subscription();
    const extensionDeactivate = typeof extensionExports.deactivate === 'function' ? extensionExports.deactivate() : null;
    this.extensionDeactivate.set(extensionID, async () => {
      try {
        if (extensionDeactivate) {
          await Promise.resolve(extensionDeactivate());
        }
      } finally {
        extensionSubscriptions.unsubscribe();
      }
    });
    // The behavior should be consistent for both sync and async activate functions that throw
    // errors or reject. Both cases should yield a rejected promise.
    //
    // TODO(sqs): Add timeouts to prevent long-running activate or deactivate functions from
    // significantly delaying other extensions.
    return tryCatchPromise(() => extensionExports.activate({
      subscriptions: extensionSubscriptions
    })).catch(error => {
      throw Object.assign(new Error(`error during extension ${JSON.stringify(extensionID)} activate function: ${error}`), {
        error
      });
    });
  }
  async $deactivateExtension(extensionID) {
    const deactivate = this.extensionDeactivate.get(extensionID);
    if (deactivate) {
      this.extensionDeactivate.delete(extensionID);
      return Promise.resolve(deactivate());
    }
  }
  /**
   * Deactivates all activated extensions that have "deactivate" functions. It does not wait for
   * the deactivation to finish for an extension if its deactivate function is async. If any
   * deactivate functions throw an error or reject, the error is logged and not propagated.
   *
   * There is no guarantee that extensions' deactivate functions are called or that execution
   * continues until they are finished. The JavaScript execution context may be terminated before
   * deactivation is completed.
   */
  unsubscribe() {
    for (const [extensionID, deactivate] of this.extensionDeactivate.entries()) {
      this.extensionDeactivate.delete(extensionID);
      tryCatchPromise(deactivate).catch(err => {
        console.warn(`Error deactivating extension ${JSON.stringify(extensionID)}:`, err);
      });
    }
  }
}

const getEventPayload = event => event.payload;
const getEvents = type => event => event.type === type;

/* tslint:disable */
function lookUp(tree, key) {
  if (key) {
    const parts = key.split('.');
    let node = tree;
    for (let i = 0; node && i < parts.length; i++) {
      node = node[parts[i]];
    }
    return node;
  }
}
class WebExtHostConfiguration {
  _proxy;
  _extHostOrganization;
  _barrier;
  _actual;
  constructor(proxy, extHostOrganization) {
    this._proxy = proxy;
    this._extHostOrganization = extHostOrganization;
    this._barrier = new Barrier();
    this._actual = null;
  }
  getConfigProvider() {
    return this._barrier.wait().then(_ => this._actual);
  }
  $initializeConfiguration(data) {
    this._actual = new ExtHostConfigProvider(this._proxy, this._extHostOrganization, data);
    this._barrier.open();
  }
  $acceptConfigurationChanged(data, change) {
    this._actual.$acceptConfigurationChanged(data, change);
  }
  unsubscribe() {}
}
class ExtHostConfigProvider {
  changed$ = new Subject();
  // private readonly _onDidChangeConfiguration = new Emitter<cdeops.ConfigurationChangeEvent>();
  _proxy;
  _extHostOrganization;
  _configurationScopes;
  _configuration;
  constructor(proxy, extHostOrganization, data) {
    this._proxy = proxy;
    this._extHostOrganization = extHostOrganization;
    this._configuration = ExtHostConfigProvider.parse(data);
    this._configurationScopes = data.configurationScopes;
  }
  // get onDidChangeConfiguration(): Event<cdeops.ConfigurationChangeEvent> {
  //     return this._onDidChangeConfiguration && this._onDidChangeConfiguration.event;
  // }
  onDidChangeConfiguration = this.changed$.pipe(filter(getEvents('onDidChangeConfiguration')), map(getEventPayload));
  $acceptConfigurationChanged(data, change) {
    const previous = {
      data: this._configuration.toData(),
      organization: this._extHostOrganization.organization
    };
    this._configuration = ExtHostConfigProvider.parse(data);
    this._configurationScopes = data.configurationScopes;
    // this._onDidChangeConfiguration.fire(this._toConfigurationChangeEvent(eventData));
    this.changed$.next({
      type: 'onDidChangeConfiguration',
      payload: this._toConfigurationChangeEvent(change, previous)
    });
  }
  getConfiguration(section, scope, extensionId) {
    const overrides = scopeToOverrides(scope) || {};
    const config = this._toReadonlyValue(section ? lookUp(this._configuration.getValue(null, overrides, this._extHostOrganization.organization), section) : this._configuration.getValue(null, overrides, this._extHostOrganization.organization));
    if (section) {
      this._validateConfigurationAccess(section, overrides, extensionId);
    }
    function parseConfigurationTarget(arg) {
      if (arg === undefined || arg === null) {
        return null;
      }
      if (typeof arg === 'boolean') {
        return arg ? 1 /* ConfigurationTarget.USER */ : 4 /* ConfigurationTarget.ORGANIZATION */;
      }
      switch (arg) {
        case ConfigurationTarget.Global:
          return 1 /* ConfigurationTarget.USER */;
        case ConfigurationTarget.Organization:
          return 4 /* ConfigurationTarget.ORGANIZATION */;
        case ConfigurationTarget.OrganizationResource:
          return 5 /* ConfigurationTarget.ORGANIZATION_RESOURCE */;
      }
    }
    const result = {
      has(key) {
        return typeof lookUp(config, key) !== 'undefined';
      },
      get: (key, defaultValue) => {
        this._validateConfigurationAccess(section ? `${section}.${key}` : key, overrides, extensionId);
        let result = lookUp(config, key);
        if (typeof result === 'undefined') {
          result = defaultValue;
        } else {
          let clonedConfig = undefined;
          const cloneOnWriteProxy = (target, accessor) => {
            let clonedTarget = undefined;
            const cloneTarget = () => {
              clonedConfig = clonedConfig ? clonedConfig : deepClone(config);
              clonedTarget = clonedTarget ? clonedTarget : lookUp(clonedConfig, accessor);
            };
            return isObject(target) ? new Proxy(target, {
              get: (target, property) => {
                if (typeof property === 'string' && property.toLowerCase() === 'tojson') {
                  cloneTarget();
                  return () => clonedTarget;
                }
                if (clonedConfig) {
                  clonedTarget = clonedTarget ? clonedTarget : lookUp(clonedConfig, accessor);
                  return clonedTarget[property];
                }
                const result = target[property];
                if (typeof property === 'string') {
                  return cloneOnWriteProxy(result, `${accessor}.${property}`);
                }
                return result;
              },
              set: (_target, property, value) => {
                cloneTarget();
                clonedTarget[property] = value;
                return true;
              },
              deleteProperty: (_target, property) => {
                cloneTarget();
                delete clonedTarget[property];
                return true;
              },
              defineProperty: (_target, property, descriptor) => {
                cloneTarget();
                Object.defineProperty(clonedTarget, property, descriptor);
                return true;
              }
            }) : target;
          };
          result = cloneOnWriteProxy(result, key);
        }
        return result;
      },
      update: (key, value, arg) => {
        key = section ? `${section}.${key}` : key;
        const target = parseConfigurationTarget(arg);
        if (value !== undefined) {
          return this._proxy.$updateConfigurationOption(target, key, value, overrides);
        } else {
          return this._proxy.$removeConfigurationOption(target, key, overrides);
        }
      },
      inspect: key => {
        key = section ? `${section}.${key}` : key;
        const config = deepClone(this._configuration.inspect(key, overrides, this._extHostOrganization.organization));
        if (config) {
          return {
            key,
            defaultValue: deepClone(config.policy?.value ?? config.default?.value),
            globalValue: deepClone(config.user?.value ?? config.application?.value),
            organizationValue: deepClone(config.organization?.value),
            organizationResourceValue: deepClone(config.organizationResource?.value)
          };
        }
        return undefined;
      }
    };
    if (typeof config === 'object') {
      mixin(result, config, false);
    }
    return Object.freeze(result);
  }
  _toReadonlyValue(result) {
    const readonlyProxy = target => {
      return isObject(target) ? new Proxy(target, {
        get: (target, property) => readonlyProxy(target[property]),
        set: (_target, property, _value) => {
          throw new Error(`TypeError: Cannot assign to read only property '${property}' of object`);
        },
        deleteProperty: (_target, property) => {
          throw new Error(`TypeError: Cannot delete read only property '${property}' of object`);
        },
        defineProperty: (_target, property) => {
          throw new Error(`TypeError: Cannot define property '${property}' for a readonly object`);
        },
        setPrototypeOf: _target => {
          throw new Error(`TypeError: Cannot set prototype for a readonly object`);
        },
        isExtensible: () => false,
        preventExtensions: () => true
      }) : target;
    };
    return readonlyProxy(result);
  }
  _validateConfigurationAccess(key, overrides, extensionId) {
    const scope = OVERRIDE_PROPERTY_REGEX.test(key) ? 4 /* ConfigurationScope.RESOURCE */ : this._configurationScopes[key];
    const extensionIdText = extensionId ? `[${extensionId.value}] ` : '';
    if (4 /* ConfigurationScope.RESOURCE */ === scope) {
      if (typeof overrides?.resource === 'undefined') {
        console.warn(`${extensionIdText}Accessing a resource scoped configuration without providing a resource is not expected. To get the effective value for '${key}', provide the URI of a resource or 'null' for any resource.`);
      }
      return;
    }
    if (3 /* ConfigurationScope.WINDOW */ === scope) {
      if (overrides?.resource) {
        console.warn(`${extensionIdText}Accessing a window scoped configuration for a resource is not expected. To associate '${key}' to a resource, define its scope to 'resource' in configuration contributions in 'package.json'.`);
      }
      return;
    }
  }
  _toConfigurationChangeEvent(change, previous) {
    const event = new ConfigurationChangeEvent(change, previous, this._configuration, this._extHostOrganization.organization);
    return Object.freeze({
      affectsConfiguration: (section, scope) => event.affectsConfiguration(section, scopeToOverrides(scope))
    });
  }
  static parse(data) {
    const defaultConfiguration = ExtHostConfigProvider.parseConfigurationModel(data.defaults);
    const userConfiguration = ExtHostConfigProvider.parseConfigurationModel(data.user);
    const organizationConfiguration = ExtHostConfigProvider.parseConfigurationModel(data.organization);
    const folders = Object.keys(data.resources).reduce((result, key) => {
      result.set(URI.parse(key), ExtHostConfigProvider.parseConfigurationModel(data.resources[key]));
      return result;
    }, new ResourceMap());
    return new Configuration(defaultConfiguration, new ConfigurationModel(), new ConfigurationModel(), userConfiguration, new ConfigurationModel(), organizationConfiguration, folders, new ConfigurationModel(), new ResourceMap(), false);
  }
  static parseConfigurationModel(model) {
    return new ConfigurationModel(model.contents, model.keys, model.overrides).freeze();
  }
}
function isUri(thing) {
  return thing instanceof URI;
}
function isOrganizationResource(thing) {
  return thing && thing.uri instanceof URI && (!thing.name || typeof thing.name === 'string') && (!thing.index || typeof thing.index === 'number');
}
function scopeToOverrides(scope) {
  if (isUri(scope)) {
    return {
      resource: scope
    };
  }
  // if (isResourceLanguage(scope)) {
  // 	return { resource: scope.uri, overrideIdentifier: scope.languageId };
  // }
  // if (isLanguage(scope)) {
  // 	return { overrideIdentifier: scope.languageId };
  // }
  if (isOrganizationResource(scope)) {
    return {
      resource: scope.uri
    };
  }
  if (scope === null) {
    return {
      resource: null
    };
  }
  return undefined;
}

class WebExtHostConfigurationPolicy {
  _proxy;
  _extHostOrganization;
  _barrier;
  _actual;
  constructor(proxy, extHostOrganization) {
    this._proxy = proxy;
    this._extHostOrganization = extHostOrganization;
    this._barrier = new Barrier();
    this._actual = null;
  }
  getConfigProvider() {
    return this._barrier.wait().then(_ => this._actual);
  }
  $initializeConfiguration(data) {
    this._actual = new ExtHostConfigProvider(this._proxy, this._extHostOrganization, data);
    this._barrier.open();
  }
  $acceptConfigurationChanged(data, change) {
    this._actual.$acceptConfigurationChanged(data, change);
  }
}

class WebExtHostConfigurationRole {
  _proxy;
  _extHostOrganization;
  _barrier;
  _actual;
  constructor(proxy, extHostOrganization) {
    this._proxy = proxy;
    this._extHostOrganization = extHostOrganization;
    this._barrier = new Barrier();
    this._actual = null;
  }
  getConfigProvider() {
    return this._barrier.wait().then(_ => this._actual);
  }
  $initializeConfiguration(data) {
    this._actual = new ExtHostConfigProvider(this._proxy, this._extHostOrganization, data);
    this._barrier.open();
  }
  $acceptConfigurationChanged(data, change) {
    this._actual.$acceptConfigurationChanged(data, change);
  }
}

function isResourceEqual(folderA, folderB) {
  return isEqual(folderA, folderB, !isLinux);
}
function compareOrganizationResourceByUri(a, b) {
  return isResourceEqual(a.uri, b.uri) ? 0 : compare(a.uri.toString(), b.uri.toString());
}
function compareOrganizationResourceByUriAndNameAndIndex(a, b) {
  if (a.index !== b.index) {
    return a.index < b.index ? -1 : 1;
  }
  return isResourceEqual(a.uri, b.uri) ? compare(a.name, b.name) : compare(a.uri.toString(), b.uri.toString());
}
function delta(oldResources, newResources, compare) {
  const oldSortedResources = oldResources.slice(0).sort(compare);
  const newSortedResources = newResources.slice(0).sort(compare);
  return delta$1(oldSortedResources, newSortedResources, compare);
}
class ExtHostOrganizationImpl extends OrganizationContext {
  _name;
  static toExtHostOrganization(data, previousConfirmedOrganization, previousUnconfirmedOrganization) {
    if (!data) {
      return {
        organization: null,
        added: [],
        removed: []
      };
    }
    const {
      id,
      name,
      resources
    } = data;
    const newOrganizationResources = [];
    // If we have an existing organization, we try to find the folders that match our
    // data and update their properties. It could be that an extension stored them
    // for later use and we want to keep them "live" if they are still present.
    const oldOrganization = previousConfirmedOrganization;
    if (oldOrganization) {
      resources.forEach((folderData, index) => {
        const folderUri = URI.revive(folderData.uri);
        const existingResource = ExtHostOrganizationImpl._findResource(previousUnconfirmedOrganization || previousConfirmedOrganization, folderUri);
        if (existingResource) {
          existingResource.name = folderData.name;
          existingResource.index = folderData.index;
          newOrganizationResources.push(existingResource);
        } else {
          newOrganizationResources.push({
            uri: folderUri,
            name: folderData.name,
            index
          });
        }
      });
    } else {
      newOrganizationResources.push(...resources.map(({
        uri,
        name,
        index
      }) => ({
        uri: URI.revive(uri),
        name,
        index
      })));
    }
    // make sure to restore sort order based on index
    newOrganizationResources.sort((f1, f2) => f1.index < f2.index ? -1 : 1);
    const organization = new ExtHostOrganizationImpl(id, name, newOrganizationResources);
    const {
      added,
      removed
    } = delta(oldOrganization ? oldOrganization.organizationResources : [], organization.organizationResources, compareOrganizationResourceByUri);
    return {
      organization,
      added,
      removed
    };
  }
  static _findResource(organization, folderUriToFind) {
    for (let i = 0; i < organization.resources.length; i++) {
      const folder = organization.organizationResources[i];
      if (isResourceEqual(folder.uri, folderUriToFind)) {
        return folder;
      }
    }
    return undefined;
  }
  _organizationResources = [];
  _structure = TernarySearchTree.forPaths();
  constructor(id, _name, folders) {
    super(id, folders.map(f => new OrganizationResource(f)));
    this._name = _name;
    // setup the organization folder data structure
    folders.forEach(folder => {
      this._organizationResources.push(folder);
      this._structure.set(folder.uri.toString(), folder);
    });
  }
  get name() {
    return this._name;
  }
  get organizationResources() {
    return this._organizationResources.slice(0);
  }
  getOrganizationResource(uri, resolveParent) {
    if (resolveParent && this._structure.get(uri.toString())) {
      // `uri` is a organization folder so we check for its parent
      uri = dirname(uri);
    }
    return this._structure.findSubstr(uri.toString());
  }
  resolveOrganizationResource(uri) {
    return this._structure.get(uri.toString());
  }
}
class WebExtHostOrganization {
  _onDidChangeOrganization = new Emitter();
  _proxy;
  _logService;
  _requestIdProvider;
  _barrier;
  _confirmedOrganization;
  _unconfirmedOrganization;
  // private _messageService: MainThreadMessageServiceShape;
  onDidChangeOrganization = this._onDidChangeOrganization.event;
  // private readonly _activeSearchCallbacks: ((match: IRawFileMatch2) => any)[] = [];
  constructor(
  // mainContext: IMainContext,
  proxy, data, logService, requestIdProvider) {
    this._logService = logService;
    this._requestIdProvider = requestIdProvider;
    this._barrier = new Barrier();
    this._proxy = proxy;
    // this._messageService = mainContext.getProxy(MainContext.MainThreadMessageService);
    this._confirmedOrganization = ExtHostOrganizationImpl.toExtHostOrganization(data).organization;
  }
  // --- organization ---
  get organization() {
    return this._actualOrganization;
  }
  get name() {
    return this._actualOrganization ? this._actualOrganization.name : undefined;
  }
  get _actualOrganization() {
    return this._unconfirmedOrganization || this._confirmedOrganization;
  }
  getOrganizationResources() {
    if (!this._actualOrganization) {
      return undefined;
    }
    return this._actualOrganization.organizationResources.slice(0);
  }
  async getOrganizationResources2() {
    await this._barrier.wait();
    if (!this._actualOrganization) {
      return undefined;
    }
    return this._actualOrganization.organizationResources.slice(0);
  }
  updateOrganizationResources(extension, index, deleteCount, ...organizationResourcesToAdd) {
    const validatedDistinctOrganizationResourcesToAdd = [];
    if (Array.isArray(organizationResourcesToAdd)) {
      organizationResourcesToAdd.forEach(folderToAdd => {
        if (URI.isUri(folderToAdd.uri) && !validatedDistinctOrganizationResourcesToAdd.some(f => isResourceEqual(f.uri, folderToAdd.uri))) {
          validatedDistinctOrganizationResourcesToAdd.push({
            uri: folderToAdd.uri,
            name: folderToAdd.name || basenameOrAuthority(folderToAdd.uri)
          });
        }
      });
    }
    if (!!this._unconfirmedOrganization) {
      return false; // prevent accumulated calls without a confirmed organization
    }
    if ([index, deleteCount].some(i => typeof i !== 'number' || i < 0)) {
      return false; // validate numbers
    }
    if (deleteCount === 0 && validatedDistinctOrganizationResourcesToAdd.length === 0) {
      return false; // nothing to delete or add
    }
    const currentOrganizationResources = this._actualOrganization ? this._actualOrganization.organizationResources : [];
    if (index + deleteCount > currentOrganizationResources.length) {
      return false; // cannot delete more than we have
    }
    // Simulate the updateOrganizationResources method on our data to do more validation
    const newOrganizationResources = currentOrganizationResources.slice(0);
    newOrganizationResources.splice(index, deleteCount, ...validatedDistinctOrganizationResourcesToAdd.map(f => ({
      uri: f.uri,
      name: f.name || basenameOrAuthority(f.uri),
      index: undefined
    })));
    for (let i = 0; i < newOrganizationResources.length; i++) {
      const folder = newOrganizationResources[i];
      if (newOrganizationResources.some((otherResource, index) => index !== i && isResourceEqual(folder.uri, otherResource.uri))) {
        return false; // cannot add the same folder multiple times
      }
    }
    newOrganizationResources.forEach((f, index) => f.index = index); // fix index
    const {
      added,
      removed
    } = delta(currentOrganizationResources, newOrganizationResources, compareOrganizationResourceByUriAndNameAndIndex);
    if (added.length === 0 && removed.length === 0) {
      return false; // nothing actually changed
    }
    // Trigger on main side
    if (this._proxy) {
      const extName = extension.displayName || extension.name;
      this._proxy.$updateOrganizationResources(extName, index, deleteCount, validatedDistinctOrganizationResourcesToAdd).then(null, error => {
        // in case of an error, make sure to clear out the unconfirmed organization
        // because we cannot expect the acknowledgement from the main side for this
        this._unconfirmedOrganization = undefined;
        // show error to user
        // this._messageService.$showMessage(Severity.Error, localize('updateerror', "Extension '{0}' failed to update organization folders: {1}", extName, error.toString()), { extension }, []);
      });
    }
    // Try to accept directly
    this.trySetOrganizationResources(newOrganizationResources);
    return true;
  }
  getOrganizationResource(uri, resolveParent) {
    if (!this._actualOrganization) {
      return undefined;
    }
    return this._actualOrganization.getOrganizationResource(uri, resolveParent);
  }
  async getOrganizationResource2(uri, resolveParent) {
    await this._barrier.wait();
    if (!this._actualOrganization) {
      return undefined;
    }
    return this._actualOrganization.getOrganizationResource(uri, resolveParent);
  }
  async resolveOrganizationResource(uri) {
    await this._barrier.wait();
    if (!this._actualOrganization) {
      return undefined;
    }
    return this._actualOrganization.resolveOrganizationResource(uri);
  }
  getPath() {
    // this is legacy from the days before having
    // multi-root and we keep it only alive if there
    // is just one organization folder.
    if (!this._actualOrganization) {
      return undefined;
    }
    const {
      resources
    } = this._actualOrganization;
    if (resources.length === 0) {
      return undefined;
    }
    // #54483 @Joh Why are we still using fsPath?
    return resources[0].uri.fsPath;
  }
  getRelativePath(pathOrUri, includeOrganization) {
    let resource;
    let path = '';
    if (typeof pathOrUri === 'string') {
      resource = URI.file(pathOrUri);
      path = pathOrUri;
    } else if (typeof pathOrUri !== 'undefined') {
      resource = pathOrUri;
      path = pathOrUri.fsPath;
    }
    if (!resource) {
      return path;
    }
    const folder = this.getOrganizationResource(resource, true);
    if (!folder) {
      return path;
    }
    if (typeof includeOrganization === 'undefined' && this._actualOrganization) {
      includeOrganization = this._actualOrganization.resources.length > 1;
    }
    let result = relativePath(folder.uri, resource);
    if (includeOrganization && folder.name) {
      result = `${folder.name}/${result}`;
    }
    return result;
  }
  trySetOrganizationResources(resources) {
    // Update directly here. The organization is unconfirmed as long as we did not get an
    // acknowledgement from the main side (via $acceptOrganizationData)
    if (this._actualOrganization) {
      this._unconfirmedOrganization = ExtHostOrganizationImpl.toExtHostOrganization({
        id: this._actualOrganization.id,
        name: this._actualOrganization.name,
        configuration: this._actualOrganization.configuration,
        resources
      }, this._actualOrganization).organization;
    }
  }
  $acceptOrganizationData(data) {
    const {
      organization,
      added,
      removed
    } = ExtHostOrganizationImpl.toExtHostOrganization(data, this._confirmedOrganization, this._unconfirmedOrganization);
    // Update our organization object. We have a confirmed organization, so we drop our
    // unconfirmed organization.
    this._confirmedOrganization = organization;
    this._unconfirmedOrganization = undefined;
    // Events
    this._onDidChangeOrganization.fire(Object.freeze({
      added,
      removed
    }));
  }
  saveAll(includeUntitled) {
    return this._proxy.$saveAll(includeUntitled);
  }
  resolveProxy(url) {
    return this._proxy.$resolveProxy(url);
  }
}

/* tslint:disable */
const consoleLogger = {
  error(message) {
    console.error(message);
  },
  warn(message) {
    console.warn(message);
  },
  info(message) {
    console.info(message);
  },
  log(message) {
    console.log(message);
  }
};
/**
 * Starts the extension host, which runs extensions. It is a Web Worker or other similar isolated
 * JavaScript execution context. There is exactly 1 extension hsot, and it has zeor or more
 * extensions activated (and running).
 *
 * It expects to receive a message containing {@link IInitData} from the client application as the
 * first message.
 *
 * @param transports The message reader and writer to use for communication with the client.
 * @return An unsubscribable to terminate the extension host.
 */
function startExtensionHost(transports, enableBackendConnection = false) {
  const connection = createConnection(transports, consoleLogger);
  connection.listen();
  const subscription = new Subscription();
  subscription.add(connection);
  // Wait for "initialize" message from client application before proceeding to create the
  // extension host.
  let initialized = false;
  const extensionAPI = new Promise(resolve => {
    connection.onRequest('initialize', async ([initData]) => {
      if (initialized) {
        throw new Error('extension host is already initialized');
      }
      initialized = true;
      let serverApi, remoteRPC;
      if (enableBackendConnection) {
        // graphql
        // setup websocket connection
        const apolloSubscription = new SubscriptionClient(initData.graphqlSubscriptionUrl, {
          reconnect: true
        });
        const link = new WebSocketLink(apolloSubscription);
        serverApi = new ApolloClient({
          link,
          cache: new InMemoryCache()
        });
        const remoteConnection = new BrowserRemoteRPC();
        remoteConnection.connect(initData.extensionSocketUrl, () => {
          const socket = new WebSocket(initData.extensionSocketUrl);
          socket.binaryType = 'arraybuffer';
          return socket;
        });
        remoteRPC = new RPCProtocol(remoteConnection, 2 /* HostIdentifier.BrowserExtHost */, new RPCLogger(), null);
      }
      const searchRequestIdProvider = new Counter();
      const extHostOrganization = new WebExtHostOrganization(createProxy(connection, 'organization'), initData.organization, logger, searchRequestIdProvider);
      handleRequests(connection, 'organization', extHostOrganization);
      const configuration = new WebExtHostConfiguration(createProxy(connection, 'configuration'), extHostOrganization);
      handleRequests(connection, 'configuration', configuration);
      // const configProvider = await configuration.getConfigProvider();
      const {
        unsubscribe,
        extensionAPI
      } = await initializeExtensionHost(connection, initData, remoteRPC, serverApi);
      subscription.add(unsubscribe);
      resolve(extensionAPI);
    });
  });
  return {
    unsubscribe: () => subscription.unsubscribe(),
    extensionAPI
  };
}
/**
 * Initializes the extension host using the {@link IInitData} from the client application. It is
 * called by {@link startExtensionHost} after the {@link IInitData} is received.
 *
 * The extension API is made globally avaialble to all requires/imports of the "cdeops" module
 * by other scripts running in the same JavaScript context.
 * @param connection The connection used to communicate with client.
 * @param initData The information to initialize theis extension host.
 * @return An unsubscribable to terminate the extension host.
 */
function initializeExtensionHost(connection, initData, remoteRPC, serverApi) {
  const subscriptions = new Subscription();
  // Workaround for different envs
  var globalObject = typeof window !== 'undefined' ? window : typeof self !== 'undefined' ? self : typeof global !== 'undefined' ? global : {};
  // step 1: attach organizationData to initData
  const {
    api,
    subscription: apiSubscription
  } = createExtensionAPI(initData, connection, remoteRPC, serverApi);
  subscriptions.add(apiSubscription);
  // Make `import 'cdeops'` or `require('cdeops')` return the extension API.
  globalObject.require = modulePath => {
    if (modulePath === 'cdeops/client') {
      return api;
    }
    // All other requires/imports in the extenion's code should not reach here because their JS
    // bundler should have resolved them locally.
    throw new Error(`require: module not found: ${modulePath}`);
  };
  subscriptions.add(() => {
    globalObject.require = () => {
      // Prevent callers from attempting to access the extension API after it was
      // unsubscribed.
      throw new Error(`require: Cdecode extension API was unsubscribed`);
    };
  });
  return {
    unsubscribe: () => {
      return subscriptions.unsubscribe();
    },
    extensionAPI: api
  };
}
function createExtensionAPI(initData, connection, rpcProtocol, serverApi) {
  const subscriptions = new Subscription();
  // For debugging/tests.
  const sync = () => connection.sendRequest('ping');
  connection.onRequest('ping', () => 'pong');
  const context = new ExtHostContext(createProxy(connection, 'context'));
  handleRequests(connection, 'context', context);
  const extensions = new WebExtHostExtensionService();
  subscriptions.add(extensions);
  handleRequests(connection, 'extensions', extensions);
  // const commands = new WebExtHostCommands(createProxy(connection, 'commands'));
  // subscriptions.add(commands);
  // handleRequests(connection, 'commands', commands);
  const searchRequestIdProvider = new Counter();
  const extHostOrganization = new WebExtHostOrganization(createProxy(connection, 'organization'), initData.organization, logger, searchRequestIdProvider);
  handleRequests(connection, 'organization', extHostOrganization);
  const configuration = new WebExtHostConfiguration(createProxy(connection, 'configuration'), extHostOrganization);
  handleRequests(connection, 'configuration', configuration);
  // const configProvider =  await configuration.getConfigProvider();
  const configurationRole = new WebExtHostConfigurationRole(createProxy(connection, 'configurationRole'), extHostOrganization);
  handleRequests(connection, 'configurationRole', configurationRole);
  const configurationPolicy = new WebExtHostConfigurationPolicy(createProxy(connection, 'configurationPolicy'), extHostOrganization);
  handleRequests(connection, 'configurationPolicy', configurationPolicy);
  // namespace: organization
  const organization = {
    get rootPath() {
      return extHostOrganization.getPath();
    },
    set rootPath(value) {
      throw Error('readonly');
    },
    getOrganizationResource(resource) {
      return extHostOrganization.getOrganizationResource(resource);
    },
    get organizationResources() {
      return extHostOrganization.getOrganizationResources();
    },
    get name() {
      return extHostOrganization.name;
    },
    set name(value) {
      throw Error('readonly');
    },
    updateOrganizationResources: (index, deleteCount, ...workspaceFoldersToAdd) => {
      return extHostOrganization.updateOrganizationResources( /**extension**/null, index, deleteCount || 0, ...workspaceFoldersToAdd);
    },
    onDidChangeOrganizationResources: function (listener, thisArgs, disposables) {
      return extHostOrganization.onDidChangeOrganization(listener, thisArgs, disposables);
    },
    onDidChangeConfiguration: null,
    getConfiguration(section, resource) {
      throw Error('not configured');
    },
    // onDidChangeConfiguration: configProvider.onDidChangeConfiguration,
    // getConfiguration(section?: string, resource?: cdeops.URI) {
    //     resource = arguments.length === 1 ? undefined : resource;
    //     return configProvider.getConfiguration(section, resource, /** extension.identifier **/ null);
    // },
    onDidChangeConfigurationRole: (listener, thisArgs, disposables) => {
      throw Error('not ready');
    },
    getConfigurationRole(section, resource) {
      throw Error('not ready');
    },
    onDidChangeConfigurationPolicy: (listener, thisArgs, disposables) => {
      throw Error('not ready');
    },
    getConfigurationPolicy(section, resource) {
      throw Error('not ready');
    }
  };
  if (rpcProtocol) {
    // remote connections
    // commneted as of 05/17/2019
    handleServerProxyRequests(connection, rpcProtocol, ExtHostContext$1);
    /**
     * Register to the ClientThread classes via `Connection`.
     * The request from the server side ExtHost will be directly forwarded to Client Thread.
     */
    // rpcProtocol.set(MainContext.MainThreadTreeViews, createProxy(connection, 'treeViews'));
    // rpcProtocol.set(MainContext.MainThreadLanguageFeatures, createProxy(connection, 'languageFeatures'));
  }
  const api = {
    // version: pkg.version,
    // commands: {
    //     registerCommand: (command, callback) => commands.registerCommand({ command, callback }),
    //     executeCommand: (command, ...args) => commands.executeCommand(command, args),
    // },
    configuration: {},
    organization,
    // types
    // Color: Color,
    ConfigurationTarget: ConfigurationTarget$1,
    LogLevel: LogLevel,
    // ThemeColor: ThemeColor,
    // ThemeIcon: ThemeIcon,
    RelativePattern: RelativePattern,
    URI: URI,
    // ViewColumn: ViewColumn,
    // WorkspaceEdit: WorkspaceEdit,
    internal: {
      updateContext: updates => context.updateContext(updates),
      cdeopsURL: URI.parse(initData.cdeopsURL),
      clientApplication: initData.clientApplication,
      sync,
      serverApi,
      gql
    }
  };
  return {
    api,
    subscription: subscriptions
  };
}

/**
 * The entrypoint for the JavaScript context that runs the extension host (and all extensions).
 *
 * To initialize the extension host, the parent sends it an 'initialize' message with
 * {@link InitData}.
 */
async function extensionHostMain() {
  try {
    const {
      unsubscribe
    } = startExtensionHost(createWebWorkerMessageTransports(), false /**true disable server side */);
    self.addEventListener('unload', () => unsubscribe());
  } catch (err) {
    console.error(err, 'Error starting the extension host:');
    self.close();
  }
}
extensionHostMain();
//# sourceMappingURL=main.worker.js.map
